package com.fossgalaxy.games.tbs.ai.rules;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.entity.HexagonTile;
import com.fossgalaxy.games.tbs.order.Order;
import com.fossgalaxy.games.tbs.ui.GameAction;
import com.fossgalaxy.object.annotations.ObjectDef;
import org.codetome.hexameter.core.api.CubeCoordinate;
import org.codetome.hexameter.core.api.Hexagon;

import java.util.*;

/**
 * Give entities the run away instruction
 *
 */
public class RunAwayRule extends MovementRule {
    private double threshold;


    @ObjectDef("RunAway")
    public RunAwayRule(double threshold){
        this.threshold = threshold;
    }

    @ObjectDef("RunAway")
    public RunAwayRule(){
        this(1);
    }

    @Override
    public boolean isForEntity(GameState state, Entity entity) {
        return (entity.getHealthFrac() <= threshold);
    }

    public CubeCoordinate findScareyestEnemy(Entity entity, GameState state) {
        CubeCoordinate ourPos = entity.getPos();

        Entity closestEnemy = null;
        double closestDist = Double.MAX_VALUE;

        Collection<Entity> entities = state.getEntities();
        for (Entity other : entities) {
            if (other.getOwner() == entity.getOwner()) {
                continue;
            }

            CubeCoordinate otherPos = other.getPos();
            double otherDist = state.getDistance(ourPos, otherPos);

            if (otherDist < closestDist) {
                closestEnemy = other;
                closestDist = otherDist;
            }

        }

        if (closestEnemy == null) {
            return null;
        }

        return closestEnemy.getPos();
    }

    @Override
    protected CubeCoordinate chooseTarget(Entity entity, GameState gameState) {

        CubeCoordinate target = findScareyestEnemy(entity, gameState);
        if (target == null) {
            return null;
        }

        double distance = -Double.MAX_VALUE;
        CubeCoordinate next = null;

        Collection<Hexagon<HexagonTile>> neighbors = gameState.getNeighbors(entity.getPos());
        for (Hexagon<HexagonTile> hex : neighbors) {

            //ignore the cell if something is there...
            if (gameState.getEntityAt(hex.getCubeCoordinate()) != null) {
                continue;
            }


            double myDist = gameState.getDistance(hex.getCubeCoordinate(), target);
            if (myDist > distance) {
                next = hex.getCubeCoordinate();
                distance = myDist;
            }
        }

        return next;
    }
}
